@@ -9,12 +9,15 @@ Collection Tables
 
     .. automethod:: __init__
 """
+import asyncio
+import json
 import os
 from functools import lru_cache
 from io import BytesIO
-from typing import Dict, List, Optional
+from typing import Callable, Dict, List, Optional
 from urllib.parse import urljoin
 
+import aiohttp
 import pandas as pd
 import pytz
 from tqdm import tqdm
@@ -40,12 +43,32 @@ DATA_FIELDS = [
     "slug",
     "modified",
     "entity__name",
+    "entity__id",
     "output",
     "process__output_schema",
+    "process__slug",
 ]
 
 CHUNK_SIZE = 1000
 
+# See _download_expressions function for in-depth explanation of this.
+EXP_ASYNC_CHUNK_SIZE = 50
+
+
+class TqdmWithCallable(tqdm):
+    """Tqdm class that also calls a given callable."""
+
+    def __init__(self, *args, **kwargs):
+        """Initialize class."""
+        self.callable = kwargs.pop("callable", None)
+        super().__init__(*args, **kwargs)
+
+    def update(self, n=1):
+        """Update."""
+        super().update(n=n)
+        if self.callable:
+            self.callable(self.n / self.total)
+
 
 class CollectionTables:
     """A helper class to fetch collection's expression and meta data.
@@ -74,15 +97,37 @@ class CollectionTables:
 
     """
 
-    def __init__(self, collection: Collection, cache_dir: Optional[str] = None):
+    def __init__(
+        self,
+        collection: Collection,
+        cache_dir: Optional[str] = None,
+        progress_callable: Optional[Callable] = None,
+        expression_source: Optional[str] = None,
+        expression_process_slug: Optional[str] = None,
+    ):
         """Initialize class.
 
         :param collection: collection to use
         :param cache_dir: cache directory location, if not specified system specific
                           cache directory is used
+        :param progress_callable: custom callable that can be used to report
+                                  progress. By default, progress is written to
+                                  stderr with tqdm
+        :param expression_source: Only consider samples in the
+                                  collection with specified source
+        :param expression_process_slug: Only consider samples in the
+                                        collection with specified
+                                        process slug
         """
         self.resolwe = collection.resolwe  # type: Resolwe
         self.collection = collection
+        self.expression_source = expression_source
+        self.expression_process_slug = expression_process_slug
+
+        self.check_heterogeneous_collections()
+
+        self.tqdm = TqdmWithCallable
+        self.progress_callable = progress_callable
 
         self.cache_dir = cache_dir
         if self.cache_dir is None:
@@ -92,6 +137,35 @@ class CollectionTables:
 
         self.gene_ids = []  # type: List[str]
 
+    def check_heterogeneous_collections(self):
+        """Ensure consistency among expressions."""
+        message = ""
+
+        process_slugs = sorted({d.process.slug for d in self._data})
+        if len(process_slugs) > 1:
+            message += (
+                "Expressions of all samples must be computed with the "
+                "same process. Expressions of samples in collection "
+                f"{self.collection.name} have been computed with "
+                f"{', '.join(process_slugs)}.\n"
+                "Use expression_process_slug filter in the "
+                "CollectionTable constructor.\n"
+            )
+
+        exp_sources = {d.output["source"] for d in self._data}
+        if len(exp_sources) > 1:
+            message += (
+                "Alignment of all samples must be computed with the "
+                "same genome source. Alignments of samples in "
+                f"collection {self.collection.name} have been computed "
+                f"with {', '.join(exp_sources)}.\n"
+                "Use expression_source filter in the CollectionTable "
+                "constructor.\n"
+            )
+
+        if message:
+            raise ValueError(message)
+
     @property
     @lru_cache()
     def meta(self) -> pd.DataFrame:
@@ -173,21 +247,36 @@ class CollectionTables:
 
         :return: list od Sample objects
         """
-        return list(self.collection.samples.filter(fields=SAMPLE_FIELDS))
+        sample_ids = [d.sample.id for d in self._data]
+        return list(
+            self.collection.samples.filter(id__in=sample_ids, fields=SAMPLE_FIELDS)
+        )
 
     @property
     @lru_cache()
     def _data(self) -> List[Data]:
         """Fetch data objects.
 
-        Fetch expression data objects from given collection and cache the results in
-        memory. Only the needed subset of fields is fetched.
+        Fetch expression data objects from given collection and cache
+        the results in memory. If ``expression_source``  /
+        ``expression_process_slug`` is provided also filter for that.
+        Only the needed subset of fields is fetched.
 
         :return: list of Data objects
         """
-        return list(
-            self.collection.data.filter(type="data:expression:", fields=DATA_FIELDS)
-        )
+        kwargs = {
+            "type": "data:expression:",
+            "fields": DATA_FIELDS,
+        }
+        if self.expression_process_slug:
+            kwargs["process__slug"] = self.expression_process_slug
+
+        data = list(self.collection.data.filter(**kwargs))
+
+        if self.expression_source:
+            data = [d for d in data if d.output["source"] == self.expression_source]
+
+        return data
 
     @property
     @lru_cache()
@@ -209,6 +298,8 @@ class CollectionTables:
             "fields": ["id", "modified"],
             "limit": 1,
         }
+        if self.expression_source or self.expression_process_slug:
+            kwargs["id__in"] = [d.sample.id for d in self._data]
 
         try:
             newest_sample = self.collection.samples.get(**kwargs)
@@ -247,12 +338,11 @@ class CollectionTables:
 
         :return: expression data version
         """
-        data_ids = self.collection.data.filter(type="data:expression:", fields=["id"])
-        if len(data_ids) == 0:
+        if len(self._data) == 0:
             raise ValueError(
                 f"Collection {self.collection.name} has no expression data!"
             )
-        data_ids = tuple(sorted(d.id for d in data_ids))
+        data_ids = tuple(sorted(d.id for d in self._data))
         version = str(hash(data_ids))
         return version
 
@@ -263,7 +353,12 @@ class CollectionTables:
             if data_type == META:
                 data = self._download_metadata()
             else:
-                data = self._download_expressions(data_type)
+                # XXX: When dropping support for Python 3.6 the below
+                # statement can be replaced with:
+                # data = asyncio.run(self._download_expressions(data_type))
+                loop = asyncio.get_event_loop()
+                data = loop.run_until_complete(self._download_expressions(data_type))
+
             save_pickle(data, self._cache_file(data_type))
         return data
 
@@ -272,7 +367,7 @@ class CollectionTables:
         version = (
             self._metadata_version if data_type == META else self._expression_version
         )
-        cache_file = f"{self.collection.slug}_{data_type}_{version}.pickle"
+        cache_file = f"{self.collection.slug}_{data_type}_{self.expression_source}_{self.expression_process_slug}_{version}.pickle"
         return os.path.join(self.cache_dir, cache_file)
 
     def _get_descriptors(self) -> pd.DataFrame:
@@ -315,7 +410,12 @@ class CollectionTables:
 
         id_to_name = {s.id: s.name for s in self._samples}
 
-        for relation in self.collection.relations:
+        for relation in self.collection.relations.filter():
+            # Only consider relations that include only samples in self.samples
+            relation_entities_ids = set([p["entity"] for p in relation.partitions])
+            if not relation_entities_ids.issubset({d.sample.id for d in self._data}):
+                pass
+
             relations[relation.category] = pd.Series(
                 index=relations.index, dtype="object"
             )
@@ -329,8 +429,9 @@ class CollectionTables:
                 elif partition["position"]:
                     value = partition["position"]
 
-                sample_name = id_to_name[partition["entity"]]
-                relations[relation.category][sample_name] = value
+                sample_name = id_to_name.get(partition["entity"], None)
+                if sample_name:
+                    relations[relation.category][sample_name] = value
 
         return relations
 
@@ -401,11 +502,12 @@ class CollectionTables:
             how = "right" if meta.columns.empty else "outer"
             meta = meta.merge(orange_data, how=how, left_index=True, right_index=True)
 
+        meta = meta.sort_index()
         meta.index.name = "sample_name"
 
         return meta
 
-    def _expression_file_url(self, data: Data, exp_type: str) -> str:
+    def _get_exp_uri(self, data: Data, exp_type: str) -> str:
         exp_files = data.files(field_name=exp_type)
 
         if not exp_files:
@@ -417,27 +519,72 @@ class CollectionTables:
                 f"Data {data.slug} has multiple expressions of type {exp_type}!"
             )
 
-        return urljoin(self.resolwe.url, f"data/{data.id}/{exp_files[0]}")
+        return f"{data.id}/{exp_files[0]}"
 
-    def _download_expressions(self, exp_type: str) -> pd.DataFrame:
+    def _get_exp_urls(self, uris):
+        response = self.resolwe.session.post(
+            urljoin(self.resolwe.url, "resolve_uris/"),
+            json={"uris": list(uris)},
+            auth=self.resolwe.auth,
+        )
+        response.raise_for_status()
+        return json.loads(response.content.decode("utf-8"))
+
+    async def _download_file(self, url, session, sample_name):
+        async with session.get(url) as response:
+            response.raise_for_status()
+            with BytesIO() as f:
+                f.write(await response.content.read())
+                f.seek(0)
+                df = pd.read_csv(f, sep="\t", compression="gzip")
+                df = df.set_index("Gene").T
+                df.index = [sample_name]
+        return df
+
+    async def _download_expressions(self, exp_type: str) -> pd.DataFrame:
         """Download expression files and marge them into a pandas DataFrame.
 
+        During normal download of a single file a signed url is created on AWS
+        and user is than redirected from Genialis server to the signed url.
+
+        However, this process (signing urls and redirecting) takes time.
+        To speedup things, we create a dedicated endpoint that accepts a bunch
+        of file uris and return a bunch of signed url's. All in one request.
+
+        However, these signed urls have expiration time of 60 s. In case of
+        large number of uris requested (> 100 uris) it is likely that url is
+        signed by Resolwe server and not downloaded for 60 seconds or more.
+        Therefore we split the uris in smaller chunks, namely
+        EXP_ASYNC_CHUNK_SIZE.
+
         :param exp_type: expression type
         :return: table with expression data, genes in columns, samples in rows
         """
         df_list = []
-        for data in tqdm(self._data, desc="Downloading expressions", ncols=100):
-            response = self.resolwe.session.get(
-                self._expression_file_url(data, exp_type), auth=self.resolwe.auth
-            )
-            response.raise_for_status()
-            with BytesIO() as f:
-                f.write(response.content)
-                f.seek(0)
-                df_ = pd.read_csv(f, sep="\t", compression="gzip")
-                df_ = df_.set_index("Gene").T
-                df_.index = [data._original_values["entity"]["name"]]
-                df_list.append(df_)
+        for i in self.tqdm(
+            range(0, len(self._data), EXP_ASYNC_CHUNK_SIZE),
+            desc="Downloading expressions",
+            ncols=100,
+            file=open(os.devnull, "w") if self.progress_callable else None,
+            callable=self.progress_callable,
+        ):
+            data_subset = self._data[i : i + EXP_ASYNC_CHUNK_SIZE]
+
+            # Mapping from expression file uri to sample name
+            uri_to_name = {
+                self._get_exp_uri(d, exp_type): d._original_values["entity"]["name"]
+                for d in data_subset
+            }
+
+            source_urls = self._get_exp_urls(uri_to_name.keys())
+            urls_names = [(url, uri_to_name[uri]) for uri, url in source_urls.items()]
+
+            async with aiohttp.ClientSession() as session:
+                futures = [
+                    self._download_file(url, session, name) for url, name in urls_names
+                ]
+                exps = await asyncio.gather(*futures)
+                df_list.extend(exps)
 
         df = pd.concat(df_list, axis=0).sort_index().sort_index(axis=1)
         source = self._data[0].output["source"]
@@ -469,7 +616,13 @@ class CollectionTables:
         """Download gene mapping."""
         sublists = [ids[i : i + CHUNK_SIZE] for i in range(0, len(ids), CHUNK_SIZE)]
         mapping = {}
-        for sublist in tqdm(sublists, desc="Downloading gene mapping", ncols=100):
+        for sublist in self.tqdm(
+            sublists,
+            desc="Downloading gene mapping",
+            ncols=100,
+            file=open(os.devnull, "w") if self.progress_callable else None,
+            callable=self.progress_callable,
+        ):
             features = self.resolwe.feature.filter(
                 source=source, species=species, feature_id__in=sublist
             )
