@@ -5,14 +5,7 @@ import tempfile
 
 from resdk import Resolwe
 
-from ..base import (
-    ADMIN_PASSWORD,
-    ADMIN_USERNAME,
-    URL,
-    USER_PASSWORD,
-    USER_USERNAME,
-    BaseResdkFunctionalTest,
-)
+from ..base import URL, USER_PASSWORD, USER_USERNAME, BaseResdkFunctionalTest
 
 TEST_FILES_DIR = os.path.abspath(
     os.path.normpath(os.path.join(__file__, "../../../files"))
@@ -30,8 +23,13 @@ class BaseResdkDocsFunctionalTest(BaseResdkFunctionalTest):
     genome_slug = "resdk-example-genome"
     genome_index_slug = "resdk-example-genome-index"
     annotation_slug = "resdk-example-annotation"
+    rrna_slug = "resdk-example-rrna"
+    rrna_index_slug = "resdk-example-rrna-index"
+    globin_slug = "resdk-example-globin"
+    globin_index_slug = "resdk-example-globin-index"
 
     def setUp(self):
+        super().setUp()
         self.tmpdir = tempfile.mkdtemp()
         self.original_cwd = os.getcwd()
         os.chdir(self.tmpdir)
@@ -48,6 +46,14 @@ class BaseResdkDocsFunctionalTest(BaseResdkFunctionalTest):
             self.genome_index.delete(force=True)
         if hasattr(self, "annotation"):
             self.annotation.delete(force=True)
+        if hasattr(self, "rrna"):
+            self.rrna.delete(force=True)
+        if hasattr(self, "rrna_index"):
+            self.rrna_index.delete(force=True)
+        if hasattr(self, "globin"):
+            self.globin.delete(force=True)
+        if hasattr(self, "globin_index"):
+            self.globin_index.delete(force=True)
 
     def run_tutorial_script(self, script_name, replace_lines=None):
         """Run a script from tutorial folder.
@@ -84,16 +90,16 @@ class BaseResdkDocsFunctionalTest(BaseResdkFunctionalTest):
         )
         return reads
 
-    def upload_genome(self, res):
+    def upload_genome(self, res, fasta, slug):
         genome = res.run(
             slug="upload-fasta-nucl",
             input={
-                "src": os.path.join(TEST_FILES_DIR, "genome.fasta.gz"),
+                "src": os.path.join(TEST_FILES_DIR, fasta),
                 "species": "Dictyostelium discoideum",
                 "build": "dd-05-2009",
             },
         )
-        self.set_slug_and_make_public(genome, self.genome_slug, permissions=["view"])
+        self.set_slug_and_make_public(genome, slug, permissions=["view"])
 
         return genome
 
@@ -113,16 +119,14 @@ class BaseResdkDocsFunctionalTest(BaseResdkFunctionalTest):
 
         return annotation
 
-    def create_genome_index(self, res, fasta):
+    def create_genome_index(self, res, fasta, slug):
         genome_index = res.run(
             slug="alignment-star-index",
             input={
                 "ref_seq": fasta,
             },
         )
-        self.set_slug_and_make_public(
-            genome_index, self.genome_index_slug, permissions=["view"]
-        )
+        self.set_slug_and_make_public(genome_index, slug, permissions=["view"])
 
         return genome_index
 
@@ -137,9 +141,8 @@ class BaseResdkDocsFunctionalTest(BaseResdkFunctionalTest):
 
 class TestIndex(BaseResdkDocsFunctionalTest):
     def setUp(self):
-        self.res = Resolwe(ADMIN_USERNAME, ADMIN_PASSWORD, URL)
-        self.reads = self.upload_reads(self.res)
         super().setUp()
+        self.reads = self.upload_reads(self.res)
 
     def test_index(self):
         """Test example code used in ``README.rst`` and ``index.rst``."""
@@ -151,16 +154,17 @@ class TestIndex(BaseResdkDocsFunctionalTest):
 
 class TestStart(BaseResdkDocsFunctionalTest):
     def setUp(self):
-        self.res = Resolwe(ADMIN_USERNAME, ADMIN_PASSWORD, URL)
+        super().setUp()
 
         # Create data for tests:
         self.reads = self.upload_reads(self.res)
-        self.genome = self.upload_genome(self.res)
-        self.genome_index = self.create_genome_index(self.res, self.genome)
+        self.genome = self.upload_genome(self.res, "genome.fasta.gz", self.genome_slug)
+        self.genome_index = self.create_genome_index(
+            self.res, self.genome, self.genome_index_slug
+        )
 
         # Set permissions for running processes:
         self.allow_run_process(self.res, "alignment-star")
-        super().setUp()
 
     def test_start(self):
         """Test getting started."""
@@ -175,10 +179,9 @@ class TestStart(BaseResdkDocsFunctionalTest):
 
 class TestTutorialGet(BaseResdkDocsFunctionalTest):
     def setUp(self):
-        self.res = Resolwe(ADMIN_USERNAME, ADMIN_PASSWORD, URL)
+        super().setUp()
 
         self.reads = self.upload_reads(self.res)
-        super().setUp()
 
     def test_tutorial_get(self):
         """Test tutorial-get."""
@@ -193,21 +196,33 @@ class TestTutorialGet(BaseResdkDocsFunctionalTest):
 
 class TestTutorialCreate(BaseResdkDocsFunctionalTest):
     def setUp(self):
-        self.res = Resolwe(ADMIN_USERNAME, ADMIN_PASSWORD, URL)
+        super().setUp()
 
         self.reads = self.upload_reads(self.res)
-        self.genome = self.upload_genome(self.res)
-        self.genome_index = self.create_genome_index(self.res, self.genome)
+
         self.annotation = self.upload_annotation(self.res)
 
+        self.genome = self.upload_genome(self.res, "genome.fasta.gz", self.genome_slug)
+        self.genome_index = self.create_genome_index(
+            self.res, self.genome, self.genome_index_slug
+        )
+
+        self.rrna = self.upload_genome(self.res, "rrna.fasta", self.rrna_slug)
+        self.rrna_index = self.create_genome_index(
+            self.res, self.rrna, self.rrna_index_slug
+        )
+        self.globin = self.upload_genome(self.res, "globin.fasta", self.globin_slug)
+        self.globin_index = self.create_genome_index(
+            self.res, self.globin, self.globin_index_slug
+        )
+
         # Set permissions for running processes:
         self.allow_run_process(self.res, "upload-fastq-single")
         self.allow_run_process(self.res, "alignment-star")
-        self.allow_run_process(self.res, "workflow-bbduk-star-htseq")
+        self.allow_run_process(self.res, "workflow-bbduk-star-featurecounts-qc-single")
         # Set permissions for using descriptor_schemas:
         self.allow_use_descriptor_schema(self.res, "reads")
         self.allow_use_descriptor_schema(self.res, "sample")
-        super().setUp()
 
     def test_tutorial_create(self):
         """Test tutorial-create."""
@@ -248,6 +263,10 @@ class TestTutorialResources(BaseResdkFunctionalTest):
             BaseResdkDocsFunctionalTest.genome_slug,
             BaseResdkDocsFunctionalTest.annotation_slug,
             BaseResdkDocsFunctionalTest.genome_index_slug,
+            BaseResdkDocsFunctionalTest.rrna_slug,
+            BaseResdkDocsFunctionalTest.rrna_index_slug,
+            BaseResdkDocsFunctionalTest.globin_slug,
+            BaseResdkDocsFunctionalTest.globin_index_slug,
         ]
         for data_slug in data_slugs:
             res.data.get(slug=data_slug, fields="id")
